@@ -18,102 +18,30 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-(*FIX: this should probably be somewhere else... *)
-module VarPrinter = struct
-  let names = Hashtbl.create 107
-  let name v nm = Hashtbl.add names v nm
-  let propagate_name v v' =
-    try name v' (Hashtbl.find names v) with Not_found -> ()
-  let name v nm =
-    let is_alpha c = (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') in
-    let is_num c = (c >= '0' && c <= '9') in
-    if String.length nm > 0 then begin
-      let nm = String.copy nm in
-      if not (is_alpha nm.[0]) then nm.[0] <- '_';
-      for i = 1 to String.length nm - 1 do
-        if not (is_alpha nm.[i] || is_num nm.[i]) then nm.[i] <- '_';
-      done;
-      let c = ref 0 in
-      for i = 0 to String.length nm - 1 do
-        if nm.[i] = '_' then incr c
-      done;
-      if !c < String.length nm then name v nm
-    end
+type addr = int
 
-  let reserved = Hashtbl.create 107
-
-  let add_reserved s =
-    if String.length s <= 5 then Hashtbl.replace reserved s ()
-
-  let _ =
-    List.iter add_reserved
-      ["break"; "case"; "catch"; "do"; "else"; "for"; "if"; "in"; "new";
-       "this"; "throw"; "try"; "var"; "void"; "while"; "with"; "class";
-       "enum"; "super"; "const"; "yield"; "let"]
-
-  let known = Hashtbl.create 1001
-
-  let last = ref (-1)
-
-  let c1 = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_$"
-  let c2 = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_$"
-
-  let rec format_ident x =
-    assert (x >= 0);
-    let char c x = String.make 1 (c.[x]) in
-    if x < 54 then
-       char c1 x
-    else
-      format_ident ((x - 54) / 64) ^ char c2 ((x - 54) mod 64)
-
-  let pretty = ref false
-
-  let format_var i x =
-    let s = format_ident x in
-    if !pretty then begin
-      try
-        let nm = Hashtbl.find names i in
-        Format.sprintf "%s_%s_" nm s
-      with Not_found ->
-        Format.sprintf "_%s_" s
-    end else
-      s
-
-  let rec to_string i =
-    try
-      Hashtbl.find known i
-    with Not_found ->
-      incr last;
-      let j = !last in
-      let s = format_var i j in
-      if Hashtbl.mem reserved s then
-        to_string i
-      else begin
-        Hashtbl.add known i s;
-        s
-      end
-
-  let reset () =
-    Hashtbl.clear names; Hashtbl.clear known; last := -1
-
-  let _ = reset ()
+module DebugAddr : sig
+  type dbg = private addr
+  val of_addr : addr -> dbg
+  val to_addr : dbg -> addr
+  val no : dbg
+end = struct
+  type dbg = int
+  let of_addr (x : addr) = (x : dbg)
+  let no = 0
+  let to_addr (x : dbg) = (x : addr)
 end
 
-let string_of_ident = VarPrinter.format_ident
-
-let add_reserved_name = VarPrinter.add_reserved
-
 module Var : sig
   type t
+
   val print : Format.formatter -> t -> unit
   val idx : t -> int
-  val to_string : t -> string
-
-  type stream
-  val make_stream : unit -> stream
-  val next : stream -> t * stream
+  val of_idx : int -> t
+  val to_string : ?origin:t -> t -> string
 
   val fresh : unit -> t
+  val fork : t -> t
 
   val count : unit -> int
 
@@ -121,56 +49,49 @@ module Var : sig
 
   val name : t -> string -> unit
   val propagate_name : t -> t -> unit
-  val set_pretty : unit -> unit
 
   val reset : unit -> unit
+  val set_pretty : bool -> unit
 
   val dummy : t
 end = struct
 
-  type t = int * int
+  open Util
+  type t = int
+
+  let printer = VarPrinter.create ()
 
   let last_var = ref 0
 
-  let reset () = last_var := 0; VarPrinter.reset ()
+  let reset () =
+    last_var := 0;
+    VarPrinter.reset printer
 
-  type stream = int
+  let to_string ?origin i = VarPrinter.to_string printer ?origin i
 
-  let c = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_$"
+  let print f x = Format.fprintf f "v%d" x
+    (* Format.fprintf f "%s" (to_string x) *)
 
-  let rec format_var x =
-    let char x = String.make 1 (c.[x]) in
-    if x < 65 then
-       char (x - 1)
-    else
-      format_var (x / 64) ^ char (x mod 64)
-
-(*
-  let to_string (x, i) = "o$" ^ format_var i(*format_var x ^ Format.sprintf "%d" i*)
-*)
-  let to_string (x, i) = VarPrinter.to_string i
-
-  let print f x = Format.fprintf f "%s" (to_string x)
-
-  let make_stream () = 1
-
-  let next current =
-    incr last_var;
-    ((current, !last_var), current + 1)
-
-  let fresh () = incr last_var; (0, !last_var)
+  let fresh () = incr last_var; !last_var
 
   let count () = !last_var + 1
 
-  let idx v = snd v
+  let idx v = v
 
-  let compare (_,v1) (_,v2) = v1 - v2
+  let of_idx v = v
 
-  let name (_, i) nm = VarPrinter.name i nm
-  let propagate_name (_, i) (_, j) = VarPrinter.propagate_name i j
-  let set_pretty () = VarPrinter.pretty := true
+  let compare v1 v2 = v1 - v2
 
-  let dummy = (-1 , -1)
+  let name i nm = VarPrinter.name printer i nm
+  let propagate_name i j = VarPrinter.propagate_name printer i j
+  let set_pretty b = VarPrinter.set_pretty printer b
+
+  let fork o =
+    let n = fresh () in
+    propagate_name o n;
+    n
+
+  let dummy = -1
 end
 
 module VarSet = Set.Make (Var)
@@ -198,7 +119,6 @@ module VarISet = struct
   let empty v = Array.make (Var.count ()) Var.dummy
 end
 
-type addr = int
 
 module AddrSet = Util.IntSet
 module AddrMap = Util.IntMap
@@ -211,25 +131,23 @@ type prim =
   | Extern of string
   | Not | IsInt
   | Eq | Neq | Lt | Le | Ult
-  | WrapInt
 
 type constant =
     String of string
+  | IString of string
   | Float of float
   | Float_array of float array
-  | Int32 of int32
-  | Nativeint of nativeint
   | Int64 of int64
   | Tuple of int * constant array
-  | Int of int
+  | Int of int32
 
 type prim_arg =
     Pv of Var.t
   | Pc of constant
 
 type expr =
-    Const of int
-  | Apply of Var.t * Var.t list * int option
+    Const of int32
+  | Apply of Var.t * Var.t list * bool
   | Block of int * Var.t array
   | Field of Var.t * int
   | Closure of Var.t list * cont
@@ -242,7 +160,7 @@ type instr =
   | Offset_ref of Var.t * int
   | Array_set of Var.t * Var.t * Var.t
 
-type cond = IsTrue | CEq of int | CLt of int | CLe of int | CUlt of int
+type cond = IsTrue | CEq of int32 | CLt of int32 | CLe of int32 | CUlt of int32
 
 type last =
     Return of Var.t
@@ -279,6 +197,8 @@ let rec print_constant f x =
   match x with
     String s ->
       Format.fprintf f "%S" s
+  | IString s ->
+      Format.fprintf f "%S" s
   | Float fl ->
       Format.fprintf f "%.12g" fl
   | Float_array a ->
@@ -288,10 +208,6 @@ let rec print_constant f x =
         Format.fprintf f "%.12g" a.(i)
       done;
       Format.fprintf f "|]"
-  | Int32 i ->
-      Format.fprintf f "%ldl" i
-  | Nativeint i ->
-      Format.fprintf f "%ndn" i
   | Int64 i ->
       Format.fprintf f "%LdL" i
   | Tuple (tag, a) ->
@@ -307,8 +223,7 @@ let rec print_constant f x =
           done;
           Format.fprintf f ")"
       end
-   | Int i ->
-       Format.fprintf f "%d" i
+  | Int i -> Format.fprintf f "%ld" i
 
 let print_arg f a =
   match a with
@@ -361,15 +276,14 @@ let print_prim f p l =
   | Lt,  [x; y]       -> Format.fprintf f "%a < %a" print_arg x print_arg y
   | Le,  [x; y]       -> Format.fprintf f "%a <= %a" print_arg x print_arg y
   | Ult, [x; y]       -> Format.fprintf f "%a <= %a" print_arg x print_arg y
-  | WrapInt, [x]      -> Format.fprintf f "to_int(%a)" print_arg x
   | _                 -> assert false
 
 let print_expr f e =
   match e with
     Const i ->
-      Format.fprintf f "%d" i
-  | Apply (g, l, n) ->
-      if n = Some (List.length l) then
+      Format.fprintf f "%ld" i
+  | Apply (g, l, exact) ->
+      if exact then
         Format.fprintf f "%a!(%a)" Var.print g print_var_list l
       else
         Format.fprintf f "%a(%a)" Var.print g print_var_list l
@@ -402,10 +316,10 @@ let print_instr f i =
 let print_cond f (c, x) =
   match c with
     IsTrue -> Var.print f x
-  | CEq n  -> Format.fprintf f "%d = %a" n Var.print x
-  | CLt n  -> Format.fprintf f "%d < %a" n Var.print x
-  | CLe n  -> Format.fprintf f "%d <= %a" n Var.print x
-  | CUlt n -> Format.fprintf f "%d < %a" n Var.print x
+  | CEq n  -> Format.fprintf f "%ld = %a" n Var.print x
+  | CLt n  -> Format.fprintf f "%ld < %a" n Var.print x
+  | CLe n  -> Format.fprintf f "%ld <= %a" n Var.print x
+  | CUlt n -> Format.fprintf f "%ld < %a" n Var.print x
 
 let print_last f l =
   match l with
@@ -471,6 +385,20 @@ let fold_closures (pc, blocks, _) f accu =
 
 (****)
 
+let prepend (start, blocks, free_pc) body =
+  let new_start = free_pc in
+  let blocks =
+    AddrMap.add new_start
+      { params = [];
+        handler = None;
+        body = body;
+        branch = Branch (start, []) }
+      blocks
+  in
+  let free_pc = free_pc + 1 in
+  (new_start, blocks, free_pc)
+
+
 let (>>) x f = f x
 
 let fold_children blocks pc f accu =
@@ -491,7 +419,7 @@ let fold_children blocks pc f accu =
       accu >> Array.fold_right (fun (pc, _) accu -> f pc accu) a1
            >> Array.fold_right (fun (pc, _) accu -> f pc accu) a2
 
-let eq (pc1,blocks1,_) (pc2,blocks2,_) =
+let eq (pc1, blocks1, _) (pc2, blocks2, _) =
   pc1 = pc2 &&
   AddrMap.cardinal blocks1 = AddrMap.cardinal blocks2 &&
   AddrMap.fold (fun pc block1 b ->
@@ -501,5 +429,5 @@ let eq (pc1,blocks1,_) (pc2,blocks2,_) =
       block1.params = block2.params &&
       block1.branch = block2.branch &&
       block1.body   = block2.body
-    with _ -> false
+    with Not_found -> false
   ) blocks1 true
