@@ -20,9 +20,6 @@
 
 open Code
 
-let inline_disabled = Util.disabled "inline"
-(****)
-
 let get_closures (_, blocks, _) =
   AddrMap.fold
     (fun _ block closures ->
@@ -43,9 +40,9 @@ let rewrite_block (pc', handler) pc blocks =
   assert (block.handler = None);
   let block = { block with handler = handler } in
   let block =
-    match block.branch with
-      Return y -> { block with branch = Branch (pc', [y]) }
-    | _        -> block
+    match block.branch, pc' with
+      Return y, Some pc' -> { block with branch = Branch (pc', [y]) }
+    | _                  -> block
   in
   AddrMap.add pc block blocks
 
@@ -101,19 +98,24 @@ let inline closures live_vars blocks free_pc pc =
     List.fold_right
       (fun i (rem, state) ->
          match i with
-           Let (x, Apply (f, args, Some n))
-               when n = List.length args
-                 && live_vars.(Var.idx f) = 1
+           Let (x, Apply (f, args, true))
+               when live_vars.(Var.idx f) = 1
                  && VarMap.mem f closures ->
              let (params, (clos_pc, clos_args)) = VarMap.find f closures in
              let (branch, blocks, free_pc) = state in
-             let blocks =
-               AddrMap.add free_pc
-                 { params = [x]; handler = block.handler;
-                   body = rem; branch = branch } blocks
+             let (blocks, cont_pc) =
+               match rem, branch with
+                 [], Return y when Var.compare x y = 0 ->
+                   (* We do not need a continuation block for tail calls *)
+                   (blocks, None)
+               | _ ->
+                   (AddrMap.add free_pc
+                      { params = [x]; handler = block.handler;
+                        body = rem; branch = branch } blocks,
+                    Some free_pc)
              in
              let blocks =
-               rewrite_closure blocks free_pc clos_pc block.handler in
+               rewrite_closure blocks cont_pc clos_pc block.handler in
              (* We do not really need this intermediate block.  It
                 just avoid the need to find which function parameters
                 are used in the function body. *)
@@ -123,6 +125,7 @@ let inline closures live_vars blocks free_pc pc =
                    body = []; branch = Branch (clos_pc, clos_args) } blocks
              in
              ([], (Branch (free_pc + 1, args), blocks, free_pc + 2))
+
          | _ ->
              (i :: rem, state))
       block.body ([], (block.branch, blocks, free_pc))
@@ -131,18 +134,12 @@ let inline closures live_vars blocks free_pc pc =
 
 (****)
 
-(*FIX: this is unefficient, as we still perform the other
-  optimizations phases repeatedly *)
-
 let f ((pc, blocks, free_pc) as p) live_vars =
-  if not (inline_disabled() || Deadcode.disabled ()) then begin
-    let closures = get_closures p in
-    let (blocks, free_pc) =
-      AddrMap.fold
-        (fun pc _ (blocks, free_pc) ->
-           inline closures live_vars blocks free_pc pc)
-        blocks (blocks, free_pc)
-    in
-    (pc, blocks, free_pc)
-  end else
-    p
+  let closures = get_closures p in
+  let (blocks, free_pc) =
+    AddrMap.fold
+      (fun pc _ (blocks, free_pc) ->
+        inline closures live_vars blocks free_pc pc)
+      blocks (blocks, free_pc)
+  in
+  (pc, blocks, free_pc)
