@@ -20,13 +20,6 @@
 
 open Javascript
 
-val eplus_int : expression -> expression -> expression
-val eminus_int : expression -> expression -> expression
+val if_statement : expression -> ?pc:node_pc -> statement -> bool -> statement -> bool -> statement list
 
-val source_elements : statement_list -> source_elements
-
-val statement_list : statement_list -> statement_list
-
-val block : statement_list -> statement
-
-val if_statement : expression -> statement -> bool -> statement -> bool -> statement list
+val get_variable : Code.VarSet.t -> expression -> Code.VarSet.t
