@@ -19,15 +19,24 @@
  *)
 
 val is_pure : string -> bool
-
-val mark_used : string -> unit
-val list_used : unit -> unit
-val get_used : unit -> string list
+val exists : string -> bool
 
 type kind = [ `Pure | `Mutable | `Mutator ]
+type kind_arg = [`Shallow_const | `Const | `Mutable]
+type t =
+  [ `Requires of Parse_info.t option * string list
+  | `Provides of Parse_info.t option * string * kind * kind_arg list option
+  | `Version of Parse_info.t option * ((int -> int -> bool) * string) list ]
 
 val kind : string -> kind
-val register : string -> kind -> unit
+val kind_args : string -> kind_arg list option
+val register : string -> kind -> kind_arg list option -> int option -> unit
+
+val arity : string -> int
 
 val alias : string -> string -> unit
 val resolve : string -> string
+
+val add_external : string -> unit
+val is_external : string -> bool
+val get_external : unit -> Util.StringSet.t
