@@ -107,7 +107,7 @@ class type style = object
   method buttonColor : Js.js_string Js.t Js.optdef Js.readonly_prop
 end
 
-let style : style Js.t = option (Js.Unsafe.variable "window.hyp_style")
+let style : style Js.t = option (Js.Unsafe.global##hyp_style)
 
 class type messages = object
   method info : Js.js_string Js.t Js.optdef Js.readonly_prop
@@ -241,6 +241,12 @@ let http_get url =
   then Lwt.return msg
   else fst (Lwt.wait ())
 
+let getfile f =
+  try
+    Lwt.return (Sys_js.file_content f)
+  with Not_found ->
+    http_get f
+
 let load_image src =
   let img = Html.createImg Html.document in
   lwt_wrap
@@ -470,17 +476,10 @@ let text_size font txt =
   res
 
 (******)
-
-class type navigator = object
-  method language : Js.js_string Js.t Js.optdef Js.readonly_prop
-  method userLanguage : Js.js_string Js.t Js.optdef Js.readonly_prop
-end
-
-let navigator : navigator Js.t = Js.Unsafe.variable "window.navigator"
-
+    
 let default_language () =
-  (Js.Optdef.get (navigator##language) (fun () ->
-   Js.Optdef.get (navigator##userLanguage) (fun () ->
+  (Js.Optdef.get (Dom_html.window##navigator##language) (fun () ->
+   Js.Optdef.get (Dom_html.window##navigator##userLanguage) (fun () ->
    Js.string "en")))##substring(0,2)
 
 let language =
@@ -497,7 +496,7 @@ let set_language lang =
   language := lang
 
 let load_messages () =
-  http_get "messages.json" >>= fun s ->
+  getfile "messages.json" >>= fun s ->
   Lwt.return (json##parse (Js.string s))
 
 let local_messages msgs : messages Js.t = option (Js.Unsafe.get msgs !language)
@@ -748,7 +747,7 @@ let rec randomize_tree n =
   let Node (info, ch) = n in
   for i = Array.length ch - 1 downto 0 do
     let v = ch.(i) in
-    let j = truncate (Js.to_float (Js.math##random()) *. float (i + 1)) in
+    let j = truncate (Js.math##random() *. float (i + 1)) in
     ch.(i) <- ch.(j);
     ch.(j) <- v
   done;
@@ -829,7 +828,7 @@ let compute_text_nodes node_names nodes =
   Html.document##title <- Js.string
     (try Hashtbl.find names "<TITLE>" with Not_found -> "");
   for i = 0 to Array.length nodes - 1 do
-    match nodes.(i) with 
+    match nodes.(i) with
       (neigh, `Txt (is_root, _, info)) ->
         let canvas =
           try
@@ -1014,7 +1013,7 @@ let tree_layout node_names root =
   (vertices, edges, nodes, boxes)
 
 let load_tree () =
-  http_get tree_url >>= fun s ->
+  getfile tree_url >>= fun s ->
   let info
     : Js.js_string Js.t tree *
       (Js.js_string Js.t * (Js.js_string Js.t * Js.js_string Js.t) array *
@@ -1049,7 +1048,7 @@ type info =
     img_url : Js.js_string Js.t option }
 
 let load_image_info () : info array Lwt.t =
-  http_get "image_info.json" >>= fun s ->
+  getfile "image_info.json" >>= fun s ->
   Lwt.return (json##parse (Js.string s))
 
 let close_button over =
