@@ -22,7 +22,12 @@
 type 'a t = {
     write: Buffer.t -> 'a -> unit;
     read: Deriving_Json_lexer.lexbuf -> 'a
-  }
+}
+
+let convert t f1 f2 = {
+  write = (fun buf a -> t.write buf (f2 a));
+  read = (fun buf -> f1 (t.read buf))
+}
 
 let to_string t v =
   let buf = Buffer.create 50 in
@@ -61,6 +66,14 @@ module type Json_min'' = sig
   val t : a t
 end
 
+module type Json_converter = sig
+  type a
+  type b
+  val t : a t
+  val from_ : a -> b
+  val to_ : b -> a
+end
+
 module type Json = sig
   type a
   val t: a t
@@ -106,6 +119,14 @@ module Defaults''(J : Json_min'') : Json with type a = J.a = struct
   let read_variant buf hash = assert false
 end
 
+module Convert(J : Json_converter) : Json with type a = J.b = struct
+  module Tmp : Json with type a = J.b =  Defaults''(struct
+      type a = J.b
+      let t = convert J.t J.from_ J.to_
+    end)
+  include Tmp
+end
+
 (** Predefs *)
 
 module Json_undef (T : sig type a end) = Defaults(struct
@@ -229,7 +250,7 @@ module Json_list(A : Json) = Defaults(struct
 		let x = A.read buf in
 		Deriving_Json_lexer.read_comma buf;
 		aux (x::l) (succ c)
-	    | _ -> failwith "Json_list.read: unexpected constructor."
+	    | _ -> Deriving_Json_lexer.tag_error ~typename:"list" buf
 	in
 	aux [] 0
     end)
@@ -246,7 +267,7 @@ module Json_ref(A : Json) = Defaults(struct
 	    let x = A.read buf in
 	    Deriving_Json_lexer.read_rbracket buf;
 	    ref x
-	| _ -> failwith "Json_ref.read: unexpected constructor."
+	| _ -> Deriving_Json_lexer.tag_error ~typename:"ref" buf
     end)
 
 module Json_option(A : Json) = Defaults(struct
@@ -265,7 +286,7 @@ module Json_option(A : Json) = Defaults(struct
 	    let x = A.read buf in
 	    Deriving_Json_lexer.read_rbracket buf;
 	    Some x
-	| _ -> failwith "Json_option.read: unexpected constructor."
+	| _ -> Deriving_Json_lexer.tag_error ~typename:"option" buf
     end)
 
 module Json_array(A : Json) = Defaults(struct
@@ -287,5 +308,5 @@ module Json_array(A : Json) = Defaults(struct
 	match Deriving_Json_lexer.read_case buf with
 	(* We allow the tag 254 in case of float array *)
 	| `NCst 0 | `NCst 254 -> Array.of_list (List.rev (read_list [] buf))
-	| _ -> failwith "Json_array.read: unexpected constructor."
+	| _ -> Deriving_Json_lexer.tag_error ~typename:"array" buf
     end)
