@@ -132,6 +132,8 @@ class type characterData = object
   method replaceData : int -> int -> js_string t -> unit meth
 end
 
+class type comment = characterData
+
 class type text = characterData
 
 class type documentFragment = node
@@ -144,6 +146,7 @@ class type ['element] document = object
   method createElementNS : js_string t -> js_string t -> 'element t meth
   method createTextNode : js_string t -> text t meth
   method createAttribute : js_string t -> attr t meth
+  method createComment : js_string t -> comment t meth
   method getElementById : js_string t -> 'element t opt meth
   method getElementsByTagName : js_string t -> 'element nodeList t meth
   method importNode : element t -> bool t -> 'element t meth
@@ -189,11 +192,11 @@ type ('a, 'b) event_listener = ('a, 'b -> bool t) meth_callback opt
 
 class type ['a] event = object
   method _type : js_string t readonly_prop
-  method target : 'a t optdef readonly_prop
-  method currentTarget : 'a t optdef readonly_prop
+  method target : 'a t opt readonly_prop
+  method currentTarget : 'a t opt readonly_prop
 
   (* Legacy methods *)
-  method srcElement : 'a t optdef readonly_prop
+  method srcElement : 'a t opt readonly_prop
 end
 
 let no_handler : ('a, 'b) event_listener = Js.null
@@ -240,10 +243,10 @@ let invoke_handler
 
 let eventTarget (e: (< .. > as 'a) #event t) : 'a t =
   let target =
-    Optdef.get (e##target) (fun () ->
-    Optdef.get (e##srcElement) (fun () -> assert false))
+    Opt.get (e##target) (fun () ->
+    Opt.get (e##srcElement) (fun () -> raise Not_found))
   in
-  if Js.instanceof target (Js.Unsafe.variable "this.Node")
+  if Js.instanceof target (Js.Unsafe.global ## _Node)
   then
     (* Workaround for Safari bug *)
     let target' : node Js.t = Js.Unsafe.coerce target in
