@@ -118,6 +118,9 @@ class type characterData = object
   method replaceData : int -> int -> js_string t -> unit meth
 end
 
+(** Specification of [Comment] objects *)
+class type comment = characterData
+
 (** Specification of [Text] objects. *)
 class type text = characterData
 
@@ -133,6 +136,7 @@ class type ['element] document = object
   method createElementNS : js_string t -> js_string t -> 'element t meth
   method createTextNode : js_string t -> text t meth
   method createAttribute : js_string t -> attr t meth
+  method createComment : js_string t -> comment t meth
   method getElementById : js_string t -> 'element t opt meth
   method getElementsByTagName : js_string t -> 'element nodeList t meth
   method importNode : element t -> bool t -> 'element t meth
@@ -189,11 +193,11 @@ type (-'a, -'b) event_listener
 
 class type ['a] event = object
   method _type : js_string t readonly_prop
-  method target : 'a t optdef readonly_prop
-  method currentTarget : 'a t optdef readonly_prop
+  method target : 'a t opt readonly_prop
+  method currentTarget : 'a t opt readonly_prop
 
   (* Legacy methods *)
-  method srcElement : 'a t optdef readonly_prop
+  method srcElement : 'a t opt readonly_prop
 end
 
 (** {2 Event handlers} *)
@@ -211,7 +215,9 @@ val invoke_handler : ('a, 'b) event_listener -> 'a -> 'b -> bool t
   (** Invoke an existing handler.  Useful to chain event handlers. *)
 
 val eventTarget : (< .. > as 'a) #event t -> 'a t
-  (** Returns which object is the target of this event. *)
+  (** Returns which object is the target of this event.
+      It raises [Not_found] in case there is no target
+      (if the event has not been triggered yet) *)
 
 type event_listener_id
 
