@@ -26,6 +26,8 @@ let onIE  = Js.to_bool (caml_js_on_ie ())
 
 external html_escape : js_string t -> js_string t = "caml_js_html_escape"
 
+external decode_html_entities : js_string t -> js_string t = "caml_js_html_entities"
+
 class type cssStyleDeclaration = object
   method background : js_string t prop
   method backgroundAttachment : js_string t prop
@@ -329,12 +331,12 @@ and element = object
 end
 
 and clientRect = object
-  method top : float t readonly_prop
-  method right : float t readonly_prop
-  method bottom : float t readonly_prop
-  method left : float t readonly_prop
-  method width : float t optdef readonly_prop
-  method height : float t optdef readonly_prop
+  method top : float readonly_prop
+  method right : float readonly_prop
+  method bottom : float readonly_prop
+  method left : float readonly_prop
+  method width : float optdef readonly_prop
+  method height : float optdef readonly_prop
 end
 
 and clientRectList = object
@@ -375,6 +377,7 @@ module Event = struct
   let hashchange = Dom.Event.make "hashchange"
   let change = Dom.Event.make "change"
   let input = Dom.Event.make "input"
+  let timeupdate = Dom.Event.make "timeupdate"
   let submit = Dom.Event.make "submit"
   let scroll = Dom.Event.make "scroll"
   let focus = Dom.Event.make "focus"
@@ -382,6 +385,7 @@ module Event = struct
   let load = Dom.Event.make "load"
   let beforeunload = Dom.Event.make "beforeunload"
   let resize = Dom.Event.make "resize"
+  let orientationchange = Dom.Event.make "orientationchange"
   let popstate = Dom.Event.make "popstate"
   let hashchange = Dom.Event.make "hashchange"
   let error = Dom.Event.make "error"
@@ -511,6 +515,7 @@ class type selectElement = object ('self)
   method remove : int -> unit meth
   method blur : unit meth
   method focus : unit meth
+  method required : bool t writeonly_prop
 
   method onchange : ('self t, event t) event_listener prop
   method oninput : ('self t, event t) event_listener prop
@@ -530,6 +535,7 @@ class type inputElement = object ('self)
   method maxLength : int prop
   method name : js_string t readonly_prop
   method readOnly : bool t prop
+  method required : bool t writeonly_prop
   method size : int prop
   method src : js_string t prop
   method tabIndex : int prop
@@ -565,6 +571,7 @@ class type textAreaElement = object ('self)
   method blur : unit meth
   method focus : unit meth
   method select : unit meth
+  method required : bool t writeonly_prop
   method placeholder : js_string t writeonly_prop
   method onselect : ('self t, event t) event_listener prop
   method onchange : ('self t, event t) event_listener prop
@@ -723,6 +730,14 @@ class type scriptElement = object
   method async : bool t prop
 end
 
+class type embedElement = object
+  inherit element
+  method src : js_string t prop
+  method height : js_string t prop
+  method width  : js_string t prop
+  method _type : js_string t prop
+end
+
 class type tableCellElement = object
   inherit element
   method cellIndex : int readonly_prop
@@ -799,7 +814,66 @@ class type tableElement = object
   method deleteRow : int -> unit meth
 end
 
-type videoElement
+class type timeRanges = object
+  method length : int readonly_prop
+  method start : int -> float meth
+  method end_ : int -> float meth
+end
+
+type networkState =
+  | NETWORK_EMPTY
+  | NETWORK_IDLE
+  | NETWORK_LOADING
+  | NETWORK_NO_SOURCE
+
+type readyState =
+  | HAVE_NOTHING
+  | HAVE_METADATA
+  | HAVE_CURRENT_DATA
+  | HAVE_FUTURE_DATA
+  | HAVE_ENOUGH_DATA
+
+(* http://www.w3schools.com/tags/ref_av_dom.asp *)
+(* only features supported by all browser. (IE9+) *)
+class type mediaElement = object
+  inherit element
+  method canPlayType : js_string t -> js_string t meth
+  method load : unit meth
+  method play : unit meth
+  method pause : unit meth
+
+  method autoplay : bool t prop
+  method buffered : timeRanges t readonly_prop
+  method controls : bool t prop
+  method currentSrc : js_string t readonly_prop
+  method currentTime : float prop
+  method duration : float readonly_prop
+  method ended : bool t readonly_prop
+  method loop : bool t prop
+  method mediagroup : js_string t prop
+  method muted : bool t prop
+  method networkState_int : int readonly_prop
+  method networkState : networkState readonly_prop
+  method paused : bool t readonly_prop
+  method playbackRate : float prop
+  method played : timeRanges t readonly_prop
+  method preload : js_string t prop
+  method readyState_int : int readonly_prop
+  method readyState : readyState readonly_prop
+  method seekable : timeRanges t readonly_prop
+  method seeking : bool t readonly_prop
+  method src : js_string t prop
+  method volume : float prop
+end
+
+class type audioElement = object
+  inherit mediaElement
+end
+
+
+class type videoElement = object
+  inherit mediaElement
+end
 
 type context = js_string t
 let _2d_ = Js.string "2d"
@@ -825,7 +899,7 @@ and canvasRenderingContext2D = object
     float -> float -> float -> float -> float -> float -> unit meth
   method setTransform :
     float -> float -> float -> float -> float -> float -> unit meth
-  method globalAlpha : float_prop
+  method globalAlpha : float prop
   method globalCompositeOperation : js_string t prop
   method strokeStyle : js_string t writeonly_prop
   method strokeStyle_gradient : canvasGradient t writeonly_prop
@@ -843,14 +917,14 @@ and canvasRenderingContext2D = object
     canvasElement t -> js_string t -> canvasPattern t meth
   method createPattern_fromVideo :
     videoElement t -> js_string t -> canvasPattern t meth
-  method lineWidth : float_prop
+  method lineWidth : float prop
   method lineCap : js_string t prop
   method lineJoin : js_string t prop
-  method miterLimit : float_prop
+  method miterLimit : float prop
 
-  method shadowOffsetX : float_prop
-  method shadowOffsetY : float_prop
-  method shadowBlur : float_prop
+  method shadowOffsetX : float prop
+  method shadowOffsetY : float prop
+  method shadowBlur : float prop
   method shadowColor : js_string t prop
 
   method clearRect : float -> float -> float -> float -> unit meth
@@ -1082,6 +1156,7 @@ class type window = object
   method onblur : (window t, event t) event_listener prop
   method onfocus : (window t, event t) event_listener prop
   method onresize : (window t, event t) event_listener prop
+  method onorientationchange : (window t, event t) event_listener prop
   method onpopstate : (window t, popStateEvent t) event_listener prop
   method onhashchange : (window t, hashChangeEvent t) event_listener prop
 
@@ -1089,7 +1164,7 @@ class type window = object
   method onoffline : (window t, event t) event_listener writeonly_prop
 end
 
-let window : window t = Js.Unsafe.variable "this" (* The toplevel object *)
+let window : window t = Js.Unsafe.global (* The toplevel object *)
 
 let document = window##document
 
@@ -1205,6 +1280,7 @@ let createOl doc : oListElement t = unsafeCreateElement doc "ol"
 let createDl doc : dListElement t = unsafeCreateElement doc "dl"
 let createLi doc : liElement t = unsafeCreateElement doc "li"
 let createDiv doc : divElement t = unsafeCreateElement doc "div"
+let createEmbed doc : embedElement t = unsafeCreateElement doc "embed"
 let createP doc : paragraphElement t = unsafeCreateElement doc "p"
 let createH1 doc : headingElement t = unsafeCreateElement doc "h1"
 let createH2 doc : headingElement t = unsafeCreateElement doc "h2"
@@ -1262,6 +1338,8 @@ let createAddress doc = createElement doc "address"
 let createFrameset doc : frameSetElement t = unsafeCreateElement doc "frameset"
 let createFrame doc : frameElement t = unsafeCreateElement doc "frame"
 let createIframe doc : iFrameElement t = unsafeCreateElement doc "iframe"
+let createAudio doc : audioElement t = unsafeCreateElement doc "audio"
+let createVideo doc : audioElement t = unsafeCreateElement doc "video"
 
 exception Canvas_not_available
 
@@ -1270,7 +1348,7 @@ let createCanvas doc : canvasElement t =
   if not (Opt.test c##getContext) then raise Canvas_not_available;
   c
 
-let html_element : htmlElement t constr = Js.Unsafe.variable "this.HTMLElement"
+let html_element : htmlElement t constr = Js.Unsafe.global ## _HTMLElement
 
 module CoerceTo = struct
   let element : #Dom.node Js.t -> element Js.t Js.opt =
@@ -1307,6 +1385,7 @@ module CoerceTo = struct
   let div e = unsafeCoerce "div" e
   let dl e = unsafeCoerce "dl" e
   let fieldset e = unsafeCoerce "fieldset" e
+  let embed e = unsafeCoerce "embed" e
   let form e = unsafeCoerce "form" e
   let frameset e = unsafeCoerce "frameset" e
   let frame e = unsafeCoerce "frame" e
@@ -1350,18 +1429,22 @@ module CoerceTo = struct
   let title e = unsafeCoerce "title" e
   let tr e = unsafeCoerce "tr" e
   let ul e = unsafeCoerce "ul" e
+  let audio e = unsafeCoerce "audio" e
+  let video e = unsafeCoerce "video" e
 
-  let unsafeCoerceEvent name (ev : #event t) =
-    let constr = Js.Unsafe.variable name in
+  let unsafeCoerceEvent constr (ev : #event t) =
     if def constr != undefined && Js.instanceof ev constr then
       Js.some (Js.Unsafe.coerce ev)
     else Js.null
 
-  let mouseEvent ev = unsafeCoerceEvent "this.MouseEvent" ev
-  let keyboardEvent ev = unsafeCoerceEvent "this.KeyboardEvent" ev
-  let wheelEvent ev = unsafeCoerceEvent "this.WheelEvent" ev
-  let mouseScrollEvent ev = unsafeCoerceEvent "this.MouseScrollEvent" ev
-  let popStateEvent ev = unsafeCoerceEvent "this.PopStateEvent" ev
+  let mouseEvent ev = unsafeCoerceEvent (Js.Unsafe.global##_MouseEvent) ev
+  let keyboardEvent ev =
+    unsafeCoerceEvent (Js.Unsafe.global##_KeyboardEvent) ev
+  let wheelEvent ev = unsafeCoerceEvent (Js.Unsafe.global##_WheelEvent) ev
+  let mouseScrollEvent ev =
+    unsafeCoerceEvent (Js.Unsafe.global##_MouseScrollEvent) ev
+  let popStateEvent ev =
+    unsafeCoerceEvent (Js.Unsafe.global##_PopStateEvent) ev
 
 end
 
@@ -1391,8 +1474,8 @@ let elementClientPosition (e : #element t) =
   let r = e##getBoundingClientRect () in
   let body = document##body in
   let html = document##documentElement in
-  (truncate (Js.to_float r##left) - body##clientLeft - html##clientLeft,
-   truncate (Js.to_float r##top) - body##clientTop - html##clientTop)
+  (truncate r##left - body##clientLeft - html##clientLeft,
+   truncate r##top - body##clientTop - html##clientTop)
 
 let getDocumentScroll () =
   let body = document##body in
@@ -1441,6 +1524,7 @@ let element : #Dom.element t -> element t = Js.Unsafe.coerce
 type taggedElement =
   | A of anchorElement t
   | Area of areaElement t
+  | Audio of audioElement t
   | Base of baseElement t
   | Blockquote of quoteElement t
   | Body of bodyElement t
@@ -1453,6 +1537,7 @@ type taggedElement =
   | Del of modElement t
   | Div of divElement t
   | Dl of dListElement t
+  | Embed of embedElement t
   | Fieldset of fieldSetElement t
   | Form of formElement t
   | Frameset of frameSetElement t
@@ -1497,6 +1582,7 @@ type taggedElement =
   | Title of titleElement t
   | Tr of tableRowElement t
   | Ul of uListElement t
+  | Video of videoElement t
   | Other of element t
 
 let other e = Other (e : #element t :> element t)
@@ -1511,6 +1597,7 @@ let tagged (e : #element t) =
         begin match tag with
         | "a" -> A (Js.Unsafe.coerce e)
         | "area" -> Area (Js.Unsafe.coerce e)
+        | "audio" -> Audio (Js.Unsafe.coerce e)
         | _ -> other e
         end
     | 'b' ->
@@ -1537,6 +1624,11 @@ let tagged (e : #element t) =
         | "dl" -> Dl (Js.Unsafe.coerce e)
         | _ -> other e
         end
+    | 'e' ->
+        begin match tag with
+        | "embed" -> Embed (Js.Unsafe.coerce e)
+        | _ -> other e
+        end
     | 'f' ->
         begin match tag with
         | "fieldset" -> Fieldset (Js.Unsafe.coerce e)
@@ -1625,6 +1717,11 @@ let tagged (e : #element t) =
         | "ul" -> Ul (Js.Unsafe.coerce e)
         | _ -> other e
         end
+    | 'v' ->
+        begin match tag with
+        | "video" -> Video (Js.Unsafe.coerce e)
+        | _ -> other e
+        end
     | _ ->
         other e
 
@@ -1675,7 +1772,7 @@ let _requestAnimationFrame : (unit -> unit) Js.callback -> unit =
          let req = List.find (fun c -> Js.Optdef.test c) l in
          fun callback -> Js.Unsafe.fun_call req [|Js.Unsafe.inject callback|]
        with Not_found ->
-         let now () = Js.to_float (jsnew Js.date_now ()##getTime()) in
+         let now () = jsnew Js.date_now ()##getTime() in
          let last = ref (now ()) in
          fun callback ->
            let t = now () in
@@ -1688,3 +1785,11 @@ let _requestAnimationFrame : (unit -> unit) Js.callback -> unit =
 
 let hasPushState () =
   Js.Optdef.test ((Js.Unsafe.coerce (window##history))##pushState)
+
+let hasPlaceholder () =
+ let i = createInput document in
+  Js.Optdef.test ((Js.Unsafe.coerce i)##placeholder)
+
+let hasRequired () =
+ let i = createInput document in
+  Js.Optdef.test ((Js.Unsafe.coerce i)##required)
