@@ -316,6 +316,8 @@ and element = object
   method dir : js_string t prop
   method className : js_string t prop
   method classList : tokenList t readonly_prop
+    (* Not supported by IE9 by default. Add +classList.js to the
+       Js_of_ocaml command line for compatibility *)
   method style : cssStyleDeclaration t prop
 
   method innerHTML : js_string t prop
@@ -344,12 +346,12 @@ end
 
 (** Rectangular box (used for element bounding boxes) *)
 and clientRect = object
-  method top : float t readonly_prop
-  method right : float t readonly_prop
-  method bottom : float t readonly_prop
-  method left : float t readonly_prop
-  method width : float t optdef readonly_prop
-  method height : float t optdef readonly_prop
+  method top : float readonly_prop
+  method right : float readonly_prop
+  method bottom : float readonly_prop
+  method left : float readonly_prop
+  method width : float optdef readonly_prop
+  method height : float optdef readonly_prop
 end
 
 and clientRectList = object
@@ -460,6 +462,7 @@ class type selectElement = object ('self)
   method remove : int -> unit meth
   method blur : unit meth
   method focus : unit meth
+  method required : bool t writeonly_prop (* Not supported by IE 9/Safari *)
 
   method onchange : ('self t, event t) event_listener prop
   method oninput : ('self t, event t) event_listener prop
@@ -479,6 +482,7 @@ class type inputElement = object ('self)
   method maxLength : int prop
   method name : js_string t readonly_prop (* Cannot be changed under IE *)
   method readOnly : bool t prop
+  method required : bool t writeonly_prop (* Not supported by IE 9/Safari *)
   method size : int prop
   method src : js_string t prop
   method tabIndex : int prop
@@ -514,6 +518,7 @@ class type textAreaElement = object ('self)
   method blur : unit meth
   method focus : unit meth
   method select : unit meth
+  method required : bool t writeonly_prop (* Not supported by IE 9/Safari *)
   method placeholder : js_string t writeonly_prop (* Not supported by IE 9 *)
 
   method onselect : ('self t, event t) event_listener prop
@@ -674,6 +679,14 @@ class type scriptElement = object
   method async : bool t prop
 end
 
+class type embedElement = object
+  inherit element
+  method src : js_string t prop
+  method height : js_string t prop
+  method width  : js_string t prop
+  method _type : js_string t prop
+end
+
 class type tableCellElement = object
   inherit element
   method cellIndex : int readonly_prop
@@ -750,7 +763,66 @@ class type tableElement = object
   method deleteRow : int -> unit meth
 end
 
-type videoElement
+class type timeRanges = object
+  method length : int readonly_prop
+  method start : int -> float meth
+  method end_ : int -> float meth
+end
+
+type networkState =
+  | NETWORK_EMPTY
+  | NETWORK_IDLE
+  | NETWORK_LOADING
+  | NETWORK_NO_SOURCE
+
+type readyState =
+  | HAVE_NOTHING
+  | HAVE_METADATA
+  | HAVE_CURRENT_DATA
+  | HAVE_FUTURE_DATA
+  | HAVE_ENOUGH_DATA
+
+class type mediaElement = object
+  inherit element
+  method canPlayType : js_string t -> js_string t meth
+  method load : unit meth
+  method play : unit meth
+  method pause : unit meth
+
+  method autoplay : bool t prop
+  method buffered : timeRanges t readonly_prop
+  method controls : bool t prop
+  method currentSrc : js_string t readonly_prop
+  method currentTime : float prop
+  method duration : float readonly_prop
+  method ended : bool t readonly_prop
+  method loop : bool t prop
+  method mediagroup : js_string t prop
+  method muted : bool t prop
+  method networkState_int : int readonly_prop
+  method networkState : networkState readonly_prop
+  method paused : bool t readonly_prop
+  method playbackRate : float prop
+  method played : timeRanges t readonly_prop
+  method preload : js_string t prop
+  method readyState_int : int readonly_prop
+  method readyState : readyState readonly_prop
+  method seekable : timeRanges t readonly_prop
+  method seeking : bool t readonly_prop
+  method src : js_string t prop
+  method volume : float prop
+end
+
+class type audioElement = object
+  inherit mediaElement
+end
+
+
+class type videoElement = object
+  inherit mediaElement
+end
+
+
 
 (** {2 Canvas object} *)
 
@@ -778,7 +850,7 @@ and canvasRenderingContext2D = object
     float -> float -> float -> float -> float -> float -> unit meth
   method setTransform :
     float -> float -> float -> float -> float -> float -> unit meth
-  method globalAlpha : float_prop
+  method globalAlpha : float prop
   method globalCompositeOperation : js_string t prop
   method strokeStyle : js_string t writeonly_prop
   method strokeStyle_gradient : canvasGradient t writeonly_prop
@@ -796,14 +868,14 @@ and canvasRenderingContext2D = object
     canvasElement t -> js_string t -> canvasPattern t meth
   method createPattern_fromVideo :
     videoElement t -> js_string t -> canvasPattern t meth
-  method lineWidth : float_prop
+  method lineWidth : float prop
   method lineCap : js_string t prop
   method lineJoin : js_string t prop
-  method miterLimit : float_prop
+  method miterLimit : float prop
 
-  method shadowOffsetX : float_prop
-  method shadowOffsetY : float_prop
-  method shadowBlur : float_prop
+  method shadowOffsetX : float prop
+  method shadowOffsetY : float prop
+  method shadowBlur : float prop
   method shadowColor : js_string t prop
 
   method clearRect : float -> float -> float -> float -> unit meth
@@ -1049,6 +1121,7 @@ class type window = object
   method onblur : (window t, event t) event_listener prop
   method onfocus : (window t, event t) event_listener prop
   method onresize : (window t, event t) event_listener prop
+  method onorientationchange : (window t, event t) event_listener prop
   method onpopstate : (window t, popStateEvent t) event_listener prop
   method onhashchange : (window t, hashChangeEvent t) event_listener prop
 
@@ -1150,6 +1223,7 @@ module Event : sig
   val hashchange : hashChangeEvent t typ
   val change : event t typ
   val input : event t typ
+  val timeupdate : event t typ
   val submit : event t typ
   val scroll : event t typ
   val focus : event t typ
@@ -1157,6 +1231,7 @@ module Event : sig
   val load : event t typ
   val beforeunload : event t typ
   val resize : event t typ
+  val orientationchange : event t typ
   val popstate : event t typ
   val hashchange : event t typ
   val error : event t typ
@@ -1244,6 +1319,7 @@ val createOl : document t -> oListElement t
 val createDl : document t -> dListElement t
 val createLi : document t -> liElement t
 val createDiv : document t -> divElement t
+val createEmbed : document t -> embedElement t
 val createP : document t -> paragraphElement t
 val createH1 : document t -> headingElement t
 val createH2 : document t -> headingElement t
@@ -1299,6 +1375,8 @@ val createAddress : document t -> element t
 val createFrameset : document t -> frameSetElement t
 val createFrame : document t -> frameElement t
 val createIframe : document t -> iFrameElement t
+val createAudio : document t -> audioElement t
+val createVideo : document t -> videoElement t
 
 exception Canvas_not_available
 val createCanvas : document t -> canvasElement t
@@ -1316,6 +1394,7 @@ val element : #Dom.element t -> element t
 type taggedElement =
   | A of anchorElement t
   | Area of areaElement t
+  | Audio of audioElement t
   | Base of baseElement t
   | Blockquote of quoteElement t
   | Body of bodyElement t
@@ -1328,6 +1407,7 @@ type taggedElement =
   | Del of modElement t
   | Div of divElement t
   | Dl of dListElement t
+  | Embed of embedElement t
   | Fieldset of fieldSetElement t
   | Form of formElement t
   | Frameset of frameSetElement t
@@ -1372,6 +1452,7 @@ type taggedElement =
   | Title of titleElement t
   | Tr of tableRowElement t
   | Ul of uListElement t
+  | Video of videoElement t
   | Other of element t
 
 val tagged : #element t -> taggedElement
@@ -1400,6 +1481,7 @@ module CoerceTo : sig
 
   val a : #element t -> anchorElement t opt
   val area : #element t -> areaElement t opt
+  val audio : #element t -> audioElement t opt
   val base : #element t -> baseElement t opt
   val blockquote : #element t -> quoteElement t opt
   val body : #element t -> bodyElement t opt
@@ -1411,6 +1493,7 @@ module CoerceTo : sig
   val colgroup : #element t -> tableColElement t opt
   val del : #element t -> modElement t opt
   val div : #element t -> divElement t opt
+  val embed : #element t -> embedElement t opt
   val dl : #element t -> dListElement t opt
   val fieldset : #element t -> fieldSetElement t opt
   val form : #element t -> formElement t opt
@@ -1456,6 +1539,7 @@ module CoerceTo : sig
   val title : #element t -> titleElement t opt
   val tr : #element t -> tableRowElement t opt
   val ul : #element t -> uListElement t opt
+  val video : #element t -> videoElement t opt
 
   (** Event *)
 
@@ -1469,5 +1553,9 @@ end
 
 (**/**)
 
+val decode_html_entities : js_string t -> js_string t
+
 val onIE : bool
 val hasPushState : unit -> bool
+val hasPlaceholder : unit -> bool
+val hasRequired : unit -> bool
