@@ -41,6 +41,7 @@ val undefined : 'a optdef
 val def : 'a -> 'a optdef
   (** Consider a value into a possibly undefined value. *)
 
+
 (** Signatures of a set of standard functions for manipulating
     optional values. *)
 module type OPT = sig
@@ -109,10 +110,6 @@ type 'a optdef_prop = <get : 'a optdef; set : 'a -> unit> gen_prop
       you can set them to a value of some type [t], but if you read
       them, you will get a value of type [t optdef] (that may be
       [undefined]). *)
-type float_prop = <get : float t; set : float -> unit> gen_prop
-  (** Type of float properties:
-      you can set them to an OCaml [float], but you will get back a
-      native Javascript number of type [float t]. *)
 
 (** {2 Object constructors} *)
 
@@ -173,7 +170,7 @@ class type js_string = object
   method toString : js_string t meth
   method valueOf : js_string t meth
   method charAt : int -> js_string t meth
-  method charCodeAt : int -> float t meth        (* This may return NaN... *)
+  method charCodeAt : int -> float meth        (* This may return NaN... *)
   method concat : js_string t -> js_string t meth
   method concat_2 : js_string t -> js_string t -> js_string t meth
   method concat_3 :
@@ -185,7 +182,7 @@ class type js_string = object
   method indexOf_from : js_string t -> int -> int meth
   method lastIndexOf : js_string t -> int meth
   method lastIndexOf_from : js_string t -> int -> int meth
-  method localeCompare : js_string t -> float t meth
+  method localeCompare : js_string t -> float meth
   method _match : regExp t -> match_result_handle t opt meth
   method replace : regExp t -> js_string t -> js_string t meth
   (* FIX: version of replace taking a function... *)
@@ -220,10 +217,10 @@ end
 
 (** Specification of the string constructor, considered as an object. *)
 class type string_constr = object
-  method fromCharCode : int -> js_string t
+  method fromCharCode : int -> js_string t meth
 end
 
-val string : string_constr t
+val string_constr : string_constr t
   (** The string constructor, as an object. *)
 
 val regExp : (js_string t -> regExp t) constr
@@ -271,7 +268,7 @@ val array_empty : 'a js_array t constr
       [jsnew array_empty ()] returns an empty array. *)
 val array_length : (int -> 'a js_array t) constr
   (** Constructor of [Array] objects.  The expression
-      [jsnew array_empty (l)] returns an array of length [l]. *)
+      [jsnew array_length (l)] returns an array of length [l]. *)
 
 val array_get : 'a #js_array t -> int -> 'a optdef
   (** Array access: [array_get a i] returns the element at index [i]
@@ -321,8 +318,8 @@ class type date = object
   method toLocaleString : js_string t meth
   method toLocaleDateString : js_string t meth
   method toLocaleTimeString : js_string t meth
-  method valueOf : float t meth
-  method getTime : float t meth
+  method valueOf : float meth
+  method getTime : float meth
   method getFullYear : int meth
   method getUTCFullYear : int meth
   method getMonth : int meth
@@ -340,69 +337,69 @@ class type date = object
   method getMilliseconds : int meth
   method getUTCMilliseconds : int meth
   method getTimezoneOffset : int meth
-  method setTime : float -> float t meth
-  method setFullYear : int -> float t meth
-  method setUTCFullYear : int -> float t meth
-  method setMonth : int -> float t meth
-  method setUTCMonth : int -> float t meth
-  method setDate : int -> float t meth
-  method setUTCDate : int -> float t meth
-  method setDay : int -> float t meth
-  method setUTCDay : int -> float t meth
-  method setHours : int -> float t meth
-  method setUTCHours : int -> float t meth
-  method setMinutes : int -> float t meth
-  method setUTCMinutes : int -> float t meth
-  method setSeconds : int -> float t meth
-  method setUTCSeconds : int -> float t meth
-  method setMilliseconds : int -> float t meth
-  method setUTCMilliseconds : int -> float t meth
+  method setTime : float -> float meth
+  method setFullYear : int -> float meth
+  method setUTCFullYear : int -> float meth
+  method setMonth : int -> float meth
+  method setUTCMonth : int -> float meth
+  method setDate : int -> float meth
+  method setUTCDate : int -> float meth
+  method setDay : int -> float meth
+  method setUTCDay : int -> float meth
+  method setHours : int -> float meth
+  method setUTCHours : int -> float meth
+  method setMinutes : int -> float meth
+  method setUTCMinutes : int -> float meth
+  method setSeconds : int -> float meth
+  method setUTCSeconds : int -> float meth
+  method setMilliseconds : int -> float meth
+  method setUTCMilliseconds : int -> float meth
   method toUTCString : js_string t meth
   method toISOString : js_string t meth
   method toJSON : 'a -> js_string t meth
 end
 
 val date_now : date t constr
-  (** Constructor of [Date] objects: [new date_now ()] returns a
+  (** Constructor of [Date] objects: [jsnew date_now ()] returns a
       [Date] object initialized with the current date. *)
 val date_fromTimeValue : (float -> date t) constr
-  (** Constructor of [Date] objects: [new date_fromTimeValue (t)] returns a
+  (** Constructor of [Date] objects: [jsnew date_fromTimeValue (t)] returns a
       [Date] object initialized with the time value [t]. *)
 val date_month : (int -> int -> date t) constr
-  (** Constructor of [Date] objects: [new date_fromTimeValue (y, m)]
+  (** Constructor of [Date] objects: [jsnew date_fromTimeValue (y, m)]
       returns a [Date] object corresponding to year [y] and month [m]. *)
 val date_day : (int -> int -> int -> date t) constr
-  (** Constructor of [Date] objects: [new date_fromTimeValue (y, m, d)]
+  (** Constructor of [Date] objects: [jsnew date_fromTimeValue (y, m, d)]
       returns a [Date] object corresponding to year [y], month [m] and
       day [d]. *)
 val date_hour : (int -> int -> int -> int -> date t) constr
-  (** Constructor of [Date] objects: [new date_fromTimeValue (y, m, d, h)]
+  (** Constructor of [Date] objects: [jsnew date_fromTimeValue (y, m, d, h)]
       returns a [Date] object corresponding to year [y] to hour [h]. *)
 val date_min : (int -> int -> int -> int -> int -> date t) constr
-  (** Constructor of [Date] objects: [new date_fromTimeValue (y, m, d, h, m')]
+  (** Constructor of [Date] objects: [jsnew date_fromTimeValue (y, m, d, h, m')]
       returns a [Date] object corresponding to year [y] to minute [m']. *)
 val date_sec : (int -> int -> int -> int -> int -> int -> date t) constr
   (** Constructor of [Date] objects:
-      [new date_fromTimeValue (y, m, d, h, m', s)]
+      [jsnew date_fromTimeValue (y, m, d, h, m', s)]
       returns a [Date] object corresponding to year [y] to second [s]. *)
 val date_ms : (int -> int -> int -> int -> int -> int -> int -> date t) constr
   (** Constructor of [Date] objects:
-      [new date_fromTimeValue (y, m, d, h, m', s, ms)]
+      [jsnew date_fromTimeValue (y, m, d, h, m', s, ms)]
       returns a [Date] object corresponding to year [y]
       to millisecond [ms]. *)
 
 (** Specification of the date constructor, considered as an object. *)
 class type date_constr = object
-  method parse : js_string t -> float t meth
-  method _UTC_month : int -> int -> float t meth
-  method _UTC_day : int -> int -> float t meth
-  method _UTC_hour : int -> int -> int -> int -> float t meth
-  method _UTC_min : int -> int -> int -> int -> int -> float t meth
-  method _UTC_sec : int -> int -> int -> int -> int -> int -> float t meth
+  method parse : js_string t -> float meth
+  method _UTC_month : int -> int -> float meth
+  method _UTC_day : int -> int -> float meth
+  method _UTC_hour : int -> int -> int -> int -> float meth
+  method _UTC_min : int -> int -> int -> int -> int -> float meth
+  method _UTC_sec : int -> int -> int -> int -> int -> int -> float meth
   method _UTC_ms :
-    int -> int -> int -> int -> int -> int -> int -> float t meth
+    int -> int -> int -> int -> int -> int -> int -> float meth
 (* This method is not available on Internet Explorer...
-  method now : float t meth
+  method now : float meth
 *)
 end
 
@@ -411,11 +408,30 @@ val date : date_constr t
 
 (** Specification of Javascript math object. *)
 class type math = object
-  method random : float t meth
+  method random : float meth
 end
 
 val math : math t
-  (** The Math object *)
+(** The Math object *)
+
+(** Specification of Javascript error object. *)
+class type error = object
+  method name : js_string t prop
+  method message : js_string t prop
+  method stack : js_string t optdef prop
+  method toString : js_string t meth
+end
+
+val error_constr : (js_string t -> error t) constr
+(** Constructor of [Error] objects:
+      [jsnew error_constr (msg)]
+      returns an [Error] object with the message [msg]. *)
+
+exception Error of error t
+  (** The [Error] exception wrap javascript exceptions when catched by ocaml code.
+      In case the javascript exception is not an instance of javascript [Error],
+      it will be serialized and wrapped into a [Failure] exception.
+  *)
 
 (** {2 Standard Javascript functions} *)
 
@@ -444,7 +460,7 @@ val isNaN : 'a -> bool
 
 val parseInt : js_string t -> int
 
-val parseFloat : js_string t -> float t
+val parseFloat : js_string t -> float
 
 
 (** {2 Conversion functions between Javascript and OCaml types} *)
@@ -459,10 +475,6 @@ external string : string -> js_string t = "caml_js_from_string"
       UTF-16.) *)
 external to_string : js_string t -> string = "caml_js_to_string"
   (** Conversion of strings from Javascript to OCaml. *)
-external float : float -> float t = "caml_js_from_float"
-  (** Conversion of OCaml floats to Javascript numbers. *)
-external to_float : float t -> float = "caml_js_to_float"
-  (** Conversion of Javascript numbers to OCaml floats. *)
 external array : 'a array -> 'a js_array t = "caml_js_from_array"
   (** Conversion of arrays from OCaml to Javascript. *)
 external to_array : 'a js_array t -> 'a array = "caml_js_to_array"
@@ -498,6 +510,13 @@ external typeof : < .. > t -> js_string t = "caml_js_typeof"
 external instanceof : < .. > t -> _ constr -> bool = "caml_js_instanceof"
   (** Tests whether a Javascript object is an instance of a given class. *)
 
+(** {2 Debugging operations.} *)
+
+external debugger : unit -> unit = "debugger"
+  (** Invokes any available debugging functionality.
+      If no debugging functionality is available, it has no effect.
+      In practice, it will insert a "debugger;" statement in the generated javascript. *)
+
 (** {2 Unsafe operations.} *)
 
 (** Unsafe Javascript operations *)
@@ -551,5 +570,22 @@ module Unsafe : sig
   external eval_string : string -> 'a = "caml_js_eval_string"
     (** Evaluate Javascript code *)
 
-(*FIX also, object/array literals *)
+  external expr : string -> 'a = "caml_js_expr"
+    (** [expr e] will parse the JavaScript expression [e] if [e]
+        is available at compile time or will failback to a
+        runtime evaluation. See [eval_string] *)
+
+  val global : < .. > t
+    (** Javascript global object *)
+
+(*FIX also, array literals *)
 end
+
+(** {2 Deprecated functions and types.} *)
+
+external float : float -> float = "%identity"
+  (** Conversion of OCaml floats to Javascript numbers. *)
+external to_float : float -> float = "%identity"
+  (** Conversion of Javascript numbers to OCaml floats. *)
+type float_prop = float prop
+  (** Type of float properties. *)
