@@ -307,21 +307,12 @@ let string_of_url = function
 module Current =
 struct
 
-  class type location = object
-    method href : Js.js_string Js.t Js.prop
-    method protocol : Js.js_string Js.t Js.readonly_prop
-    method host : Js.js_string Js.t Js.readonly_prop
-    method hostname : Js.js_string Js.t Js.readonly_prop
-    method port : Js.js_string Js.t Js.readonly_prop
-    method pathname : Js.js_string Js.t Js.readonly_prop
-    method search : Js.js_string Js.t Js.readonly_prop
-    method hash : Js.js_string Js.t Js.prop
-  end
-
-  let l : location Js.t = Js.Unsafe.variable "location"
+  let l = Dom_html.window##location
 
   let host = urldecode_js_string_string l##hostname
 
+  let protocol = urldecode_js_string_string l##protocol
+
   let port =
     try Some (int_of_string (Js.to_bytestring l##port))
     with Failure _ -> None
