@@ -46,6 +46,11 @@ class type xmlHttpRequest = object ('self)
 
   inherit File.progressEventTarget
   method ontimeout : ('self t, 'self File.progressEvent t) Dom.event_listener writeonly_prop
+  method upload : xmlHttpRequestUpload t optdef readonly_prop
+end
+
+and xmlHttpRequestUpload = object ('self)
+  inherit File.progressEventTarget
 end
 
 module Event = struct
@@ -66,17 +71,11 @@ class type xmlHttpRequest_binary = object
   method sendAsBinary_presence : unit optdef readonly_prop
 end
 
-let xmlHttpRequest () : xmlHttpRequest t constr =
-  Js.Unsafe.variable "XMLHttpRequest"
-
-let activeXObject () : (js_string t -> xmlHttpRequest t) constr =
-  Js.Unsafe.variable "ActiveXObject"
-
 let create () =
-  try jsnew (xmlHttpRequest ()) () with _ ->
-  try jsnew (activeXObject ()) (Js.string "Msxml2.XMLHTTP") with _ ->
-  try jsnew (activeXObject ()) (Js.string "Msxml3.XMLHTTP") with _ ->
-  try jsnew (activeXObject ()) (Js.string "Microsoft.XMLHTTP") with _ ->
+  try jsnew (Js.Unsafe.global##_XMLHttpRequest)() with _ ->
+  try jsnew (Js.Unsafe.global##activeXObject)(Js.string "Msxml2.XMLHTTP") with _ ->
+  try jsnew (Js.Unsafe.global##activeXObject)(Js.string "Msxml3.XMLHTTP") with _ ->
+  try jsnew (Js.Unsafe.global##activeXObject)(Js.string "Microsoft.XMLHTTP") with _ ->
   assert false
 
 let encode = Url.encode_arguments
@@ -86,7 +85,7 @@ let encode_url args =
 
 let generateBoundary () =
   let nine_digits () =
-    string_of_int (truncate (Js.to_float (Js.math##random()) *. 1000000000.))
+    string_of_int (truncate (Js.math##random() *. 1000000000.))
   in
   "js_of_ocaml-------------------" ^ nine_digits () ^ nine_digits ()
 
@@ -97,12 +96,12 @@ let encode_multipart boundary elements =
      (fun v ->
        ignore(b##push(Js.string ("--"^boundary^"\r\n")));
        match v with
-	 | name,`String value ->
+	 | name, `String value ->
 	   ignore(b##push_3(Js.string ("Content-Disposition: form-data; name=\"" ^ name ^ "\"\r\n\r\n"),
 			    value,
 			    Js.string "\r\n"));
 	   return ()
-	 | name,`File value ->
+	 | name, `File value ->
 	   File.readAsBinaryString (value :> File.blob Js.t)
 	   >>= (fun file ->
 	     ignore(b##push_4(Js.string ("Content-Disposition: form-data; name=\"" ^ name ^ "\"; filename=\""),
@@ -157,27 +156,33 @@ let extract_get_param url =
 let perform_raw_url
     ?(headers = [])
     ?content_type
-    ?(post_args:(string * string) list option)
+    ?(post_args:(string * Form.form_elt) list option)
     ?(get_args=[])
     ?(form_arg:Form.form_contents option)
     ?(check_headers=(fun _ _ -> true))
+    ?progress
+    ?upload_progress
     ?override_mime_type
     url =
 
   let form_arg =
     match form_arg with
       | None ->
-	( match post_args with
+	(match post_args with
 	  | None -> None
 	  | Some post_args ->
 	    let contents = Form.empty_form_contents () in
-	    List.iter (fun (name,value) -> Form.append contents (name,`String (string value))) post_args;
-	    Some contents )
+	    List.iter (fun (name, value) ->
+              Form.append contents (name, value))
+              post_args;
+	    Some contents)
       | Some form_arg ->
-	( match post_args with
+	(match post_args with
 	  | None -> ()
 	  | Some post_args ->
-	    List.iter (fun (name,value) -> Form.append form_arg (name,`String (string value))) post_args; );
+	    List.iter (fun (name, value) ->
+              Form.append form_arg (name, value))
+              post_args);
 	Some form_arg
   in
 
@@ -260,6 +265,24 @@ let perform_raw_url
             }
 	| _ -> ()));
 
+  begin match progress with
+  | Some progress ->
+    req##onprogress <- Dom.handler
+      (fun e ->
+        progress e##loaded e##total;
+        Js._true)
+  | None -> ()
+  end;
+  Optdef.iter (req##upload) (fun upload ->
+    match upload_progress with
+    | Some upload_progress ->
+      upload##onprogress <- Dom.handler
+        (fun e ->
+          upload_progress e##loaded e##total;
+          Js._true)
+    | None -> ()
+  );
+
   (match form_arg with
      | None -> req##send (Js.null)
      | Some (`Fields l) ->
@@ -295,4 +318,3 @@ let perform
     (Url.string_of_url url)
 
 let get s = perform_raw_url s
-
