@@ -37,7 +37,8 @@ function caml_js_typeof(o) { return typeof o; }
 
 //Provides: caml_js_on_ie const
 function caml_js_on_ie () {
-  var ua = this.navigator?this.navigator.userAgent:"";
+  var ua =
+    joo_global_object.navigator?joo_global_object.navigator.userAgent:"";
   return ua.indexOf("MSIE") != -1 && ua.indexOf("Opera") != 0;
 }
 
@@ -50,13 +51,57 @@ function caml_js_html_escape (s) {
           .replace(caml_js_regexps.quot, "&quot;");
 }
 
+//Provides: caml_js_html_entities
+function caml_js_html_entities(s) {
+    var str, temp = document.createElement('p');
+    temp.innerHTML= s;
+    str= temp.textContent || temp.innerText;
+    temp=null;
+    return str;
+}
+
 /////////// Debugging console
 //Provides: caml_js_get_console const
 function caml_js_get_console () {
-  var c = this.console?this.console:{};
+  var c = joo_global_object.console?joo_global_object.console:{};
   var m = ["log", "debug", "info", "warn", "error", "assert", "dir", "dirxml",
            "trace", "group", "groupCollapsed", "groupEnd", "time", "timeEnd"];
   function f () {}
   for (var i = 0; i < m.length; i++) if (!c[m[i]]) c[m[i]]=f;
   return c;
 }
+
+//Provides:caml_trampoline
+function caml_trampoline(res) {
+  var c = 1;
+  while(res && res.joo_tramp){
+    res = res.joo_tramp.apply(null, res.joo_args);
+    c++;
+  }
+  //if(c>10) joo_global_object.console.log("trampoline ", c, "times")
+  return res;
+}
+
+//Provides:caml_trampoline_return
+function caml_trampoline_return(f,args) {
+  return {joo_tramp:f,joo_args:args};
+}
+
+//Provides: js_print_stdout
+function js_print_stdout(s) {
+  // Do not output the last \n if present
+  // as console logging display a newline at the end
+  if(s.charCodeAt(s.length - 1) == 10)
+    s = s.substr(0,s.length - 1 );
+  var v = joo_global_object.console;
+  v  && v.log && v.log(s);
+}
+//Provides: js_print_stderr
+function js_print_stderr(s) {
+  // Do not output the last \n if present
+  // as console logging display a newline at the end
+  if(s.charCodeAt(s.length - 1) == 10)
+    s = s.substr(0,s.length - 1 );
+  var v = joo_global_object.console;
+  v && v.error && v.error(s);
+}
