@@ -30,7 +30,7 @@ function caml_js_to_float(x) { return x; }
 //Provides: caml_js_from_string mutable
 function caml_js_from_string(s) { return s.toString(); }
 //Provides: caml_js_to_string const
-//Requires: MlString
+//Requires: MlWrappedString
 function caml_js_to_string(s) { return new MlWrappedString(s); }
 //Provides: caml_js_from_array mutable
 function caml_js_from_array(a) { return a.slice(1); }
@@ -38,10 +38,19 @@ function caml_js_from_array(a) { return a.slice(1); }
 function caml_js_to_array(a) { return [0].concat(a); }
 
 //Provides: caml_js_var mutable
-function caml_js_var(x) { return eval(x.toString()); }
+//Requires: js_print_stderr
+function caml_js_var(x) {
+  var x = x.toString();
+  //Checks that x has the form ident[.ident]*
+  if(!x.match(/^[a-zA-Z_$][a-zA-Z_$0-9]*(\.[a-zA-Z_$][a-zA-Z_$0-9]*)*$/)){
+    js_print_stderr("caml_js_var: \"" + x + "\" is not a valid JavaScript variable. continuing ..");
+    //joo_global_object.console.error("Js.Unsafe.eval_string")
+  }
+  return eval(x);
+}
 //Provides: caml_js_const const
 function caml_js_const(x) {
-  switch (caml_string_to_js(x)) {
+  switch (x.toString()) {
   case "null": return null;
   case "true": return true;
   case "false": return false;
@@ -66,7 +75,7 @@ function caml_js_new(c, a) {
   case 7: return new c (a[1],a[2],a[3],a[4],a[5],a[6]);
   case 8: return new c (a[1],a[2],a[3],a[4],a[5],a[6], a[7]);
   }
-  function F() { return c.apply(this, args.slice(1)); }
+  function F() { return c.apply(this, a.slice(1)); }
   F.prototype = c.prototype;
   return new F;
 }
@@ -94,12 +103,17 @@ function caml_js_equals (x, y) { return +(x == y); }
 //Provides: caml_js_from_byte_string mutable
 function caml_js_from_byte_string (s) {return s.getFullBytes();}
 //Provides: caml_js_to_byte_string const
+//Requires: MlString
 function caml_js_to_byte_string (s) {return new MlString (s);}
 
-//Provides: caml_js_eval
-function caml_js_eval () {return eval(arguments[0]);}
 //Provides: caml_js_eval_string
-function caml_js_eval_string () {return eval(arguments[0].toString());}
+function caml_js_eval_string (s) {return eval(s.toString());}
+
+//Provides: caml_js_expr
+//Requires: js_print_stderr
+function caml_js_expr(s) {
+  js_print_stderr("caml_js_expr: fallback to runtime evaluation");
+  return eval(s.toString());}
 
 //Provides: caml_js_object
 function caml_js_object (a) {
@@ -109,4 +123,4 @@ function caml_js_object (a) {
     o[p[1]] = p[2];
   }
   return o;
-}
\ No newline at end of file
+}
