@@ -196,7 +196,8 @@
 // Create a JSON object only if one does not already exist. We create the
 // methods in a closure to avoid creating global variables.
 
-//Provides: caml_json
+
+//Provides: JSON
 var JSON;
 if (!JSON) {
     JSON = {};
@@ -212,7 +213,7 @@ if (!JSON) {
 
     if (typeof Date.prototype.toJSON !== 'function') {
 
-        Date.prototype.toJSON = function (key) {
+        Date.prototype.toJSON = function (_key) {
 
             return isFinite(this.valueOf()) ?
                 this.getUTCFullYear()     + '-' +
@@ -225,7 +226,7 @@ if (!JSON) {
 
         String.prototype.toJSON      =
             Number.prototype.toJSON  =
-            Boolean.prototype.toJSON = function (key) {
+            Boolean.prototype.toJSON = function (_key) {
                 return this.valueOf();
             };
     }
@@ -518,4 +519,6 @@ if (!JSON) {
 
 }());
 
-function caml_json() { return JSON; }
\ No newline at end of file
+//Provides: caml_json
+//Requires: JSON
+function caml_json() { return JSON; }
