@@ -38,32 +38,50 @@ var caml_named_values = {};
 //Provides: caml_register_named_value
 //Requires: caml_named_values
 function caml_register_named_value(nm,v) {
-  caml_named_values[nm] = v; return 0;
+  caml_named_values[nm.toString()] = v; return 0;
+}
+
+//Provides: caml_named_value
+//Requires: caml_named_values
+function caml_named_value(nm) {
+  return caml_named_values[nm]
 }
 
 //Provides: caml_global_data
 var caml_global_data = [0];
 
-//Provides: caml_register_global
+//Provides: caml_register_global(const,shallow)
 //Requires: caml_global_data
 function caml_register_global (n, v) { caml_global_data[n + 1] = v; }
 
+//Provides: caml_get_global_data
+//Requires: caml_global_data
+function caml_get_global_data () { return caml_global_data; }
+
+//Raise exception
+
+
 //Provides: caml_raise_constant
+//Version: < 4.02
 function caml_raise_constant (tag) { throw [0, tag]; }
 
+//Provides: caml_raise_constant
+//Version: >= 4.02
+function caml_raise_constant (tag) { throw tag; }
+
 //Provides: caml_raise_with_arg
 function caml_raise_with_arg (tag, arg) { throw [0, tag, arg]; }
 
 //Provides: caml_raise_with_string
-//Requires: caml_raise_with_arg, MlString
+//Requires: caml_raise_with_arg,MlWrappedString
 function caml_raise_with_string (tag, msg) {
   caml_raise_with_arg (tag, new MlWrappedString (msg));
 }
 
-//Provides: caml_invalid_argument
-//Requires: caml_raise_with_string
-function caml_invalid_argument (msg) {
-  caml_raise_with_string(caml_global_data[4], msg);
+//Provides: caml_raise_sys_error
+//Requires: caml_raise_with_string, caml_global_data
+function caml_raise_sys_error (msg) {
+  caml_raise_with_string(caml_global_data[2], msg);
 }
 
 //Provides: caml_failwith
@@ -71,11 +89,39 @@ function caml_invalid_argument (msg) {
 function caml_failwith (msg) {
   caml_raise_with_string(caml_global_data[3], msg);
 }
+//Provides: caml_wrap_exception
+//Requires: caml_global_data,MlWrappedString,caml_named_value
+function caml_wrap_exception(e) {
+  if(e instanceof Array) return e;
+  //Stack_overflow: chrome, safari
+  if(joo_global_object.RangeError
+     && e instanceof joo_global_object.RangeError
+     && e.message
+     && e.message.match(/maximum call stack/i))
+    return [0,caml_global_data[9]];
+  //Stack_overflow: firefox
+  if(joo_global_object.InternalError
+     && e instanceof joo_global_object.InternalError
+     && e.message
+     && e.message.match(/too much recursion/i))
+    return [0,caml_global_data[9]];
+  //Wrap Error in Js.Error exception
+  if(e instanceof joo_global_object.Error)
+    return [0,caml_named_value("jsError"),e];
+  //fallback: wrapped in Failure
+  return [0,caml_global_data[3],new MlWrappedString (String(e))];
+}
 
-//Provides: caml_array_bound_error
-//Requires: caml_invalid_argument
-function caml_array_bound_error () {
-  caml_invalid_argument("index out of bounds");
+//Provides: caml_invalid_argument
+//Requires: caml_raise_with_string, caml_global_data
+function caml_invalid_argument (msg) {
+  caml_raise_with_string(caml_global_data[4], msg);
+}
+
+//Provides: caml_raise_end_of_file
+//Requires: caml_raise_constant, caml_global_data
+function caml_raise_end_of_file () {
+  caml_raise_constant(caml_global_data[5]);
 }
 
 //Provides: caml_raise_zero_divide
@@ -86,7 +132,15 @@ function caml_raise_zero_divide () {
 
 //Provides: caml_raise_not_found
 //Requires: caml_raise_constant, caml_global_data
-function caml_raise_not_found () { caml_raise_constant(caml_global_data[7]); }
+function caml_raise_not_found () {
+  caml_raise_constant(caml_global_data[7]); }
+
+
+//Provides: caml_array_bound_error
+//Requires: caml_invalid_argument
+function caml_array_bound_error () {
+  caml_invalid_argument("index out of bounds");
+}
 
 //Provides: caml_update_dummy
 function caml_update_dummy (x, y) {
@@ -116,9 +170,10 @@ function caml_obj_truncate (x, s) { x.length = s + 1; return 0; }
 function caml_lazy_make_forward (v) { return [250, v]; }
 
 //Provides: caml_mul const
-function caml_mul(x,y) {
-  return ((((x >> 16) * y) << 16) + (x & 0xffff) * y)|0;
-}
+if (!Math.imul)
+  Math.imul =
+    function (x,y) { return ((((x >> 16) * y) << 16) + (x & 0xffff) * y)|0; };
+var caml_mul = Math.imul;
 
 //slightly slower
 // function mul32(x,y) {
@@ -163,13 +218,14 @@ function caml_make_vect (len, init) {
 
 //Provides: caml_compare_val
 //Requires: MlString, caml_int64_compare, caml_int_compare
+//Requires: caml_invalid_argument
 function caml_compare_val (a, b, total) {
   var stack = [];
   for(;;) {
     if (!(total && a === b)) {
       if (a instanceof MlString) {
         if (b instanceof MlString) {
-            if (a != b) {
+            if (a !== b) {
 		var x = a.compare(b);
 		if (x != 0) return x;
 	    }
@@ -177,14 +233,18 @@ function caml_compare_val (a, b, total) {
           // Should not happen
           return 1;
       } else if (a instanceof Array && a[0] === (a[0]|0)) {
-        // Forward object
         var ta = a[0];
+        // ignore double_array_tag
+        if (ta === 254) ta=0;
+        // Forward object
         if (ta === 250) {
           a = a[1];
           continue;
         } else if (b instanceof Array && b[0] === (b[0]|0)) {
-          // Forward object
           var tb = b[0];
+          // ignore double_array_tag
+          if (tb === 254) tb=0;
+          // Forward object
           if (tb === 250) {
             b = b[1];
             continue;
@@ -198,6 +258,9 @@ function caml_compare_val (a, b, total) {
 		if (x != 0) return x;
 		break;
 	    }
+            case 251: {
+                caml_invalid_argument("equal: abstract value");
+            }
             case 255: {
 		// Int64
 		var x = caml_int64_compare(a, b);
@@ -214,10 +277,13 @@ function caml_compare_val (a, b, total) {
       } else if (b instanceof MlString ||
                  (b instanceof Array && b[0] === (b[0]|0))) {
         return -1;
+      } else if (typeof a != "number" && a && a.compare) {
+        return a.compare(b,total);
       } else {
         if (a < b) return -1;
         if (a > b) return 1;
-        if (total && a != b) {
+        if (a != b) {
+          if (!total) return NaN;
           if (a == a) return 1;
           if (b == b) return -1;
         }
@@ -243,20 +309,20 @@ function caml_int_compare (a, b) {
 //Requires: caml_compare_val
 function caml_equal (x, y) { return +(caml_compare_val(x,y,false) == 0); }
 //Provides: caml_notequal mutable
-//Requires: caml_compare
+//Requires: caml_compare_val
 function caml_notequal (x, y) { return +(caml_compare_val(x,y,false) != 0); }
 //Provides: caml_greaterequal mutable
-//Requires: caml_compare
-function caml_greaterequal (x, y) { return +(caml_compare(x,y,false) >= 0); }
+//Requires: caml_compare_val
+function caml_greaterequal (x, y) { return +(caml_compare_val(x,y,false) >= 0); }
 //Provides: caml_greaterthan mutable
-//Requires: caml_compare
-function caml_greaterthan (x, y) { return +(caml_compare(x,y,false) > 0); }
+//Requires: caml_compare_val
+function caml_greaterthan (x, y) { return +(caml_compare_val(x,y,false) > 0); }
 //Provides: caml_lessequal mutable
-//Requires: caml_compare
-function caml_lessequal (x, y) { return +(caml_compare(x,y,false) <= 0); }
+//Requires: caml_compare_val
+function caml_lessequal (x, y) { return +(caml_compare_val(x,y,false) <= 0); }
 //Provides: caml_lessthan mutable
-//Requires: caml_compare
-function caml_lessthan (x, y) { return +(caml_compare(x,y,false) < 0); }
+//Requires: caml_compare_val
+function caml_lessthan (x, y) { return +(caml_compare_val(x,y,false) < 0); }
 
 //Provides: caml_parse_sign_and_base
 //Requires: MlString
@@ -299,9 +365,14 @@ function caml_int_of_string (s) {
     if (res > threshold) caml_failwith("int_of_string");
   }
   if (i != s.getLen()) caml_failwith("int_of_string");
+  // For base different from 10, we expect an unsigned representation,
+  // hence any value of 'res' (less than 'threshold') is acceptable.
+  // But we have to convert the result back to a signed integer.
   res = sign * res;
-  if ((res | 0) != res) caml_failwith("int_of_string");
-  return res;
+  if ((base == 10) && ((res | 0) != res))
+    /* Signed representation expected, allow -2^(nbits-1) to 2^(nbits-1) - 1 */
+    caml_failwith("int_of_string");
+  return res | 0;
 }
 
 //Provides: caml_float_of_string mutable
@@ -314,6 +385,13 @@ function caml_float_of_string(s) {
   s = s.replace(/_/g,"");
   res = +s;
   if (((s.length > 0) && (res === res)) || /^[+-]?nan$/i.test(s)) return res;
+  if(/^ *0x[0-9a-f_]+p[+-]?[0-9_]+/i.test(s)){
+    var pidx = s.indexOf('p');
+    pidx = (pidx==-1)?s.indexOf('P'):pidx;
+    var exp = +s.substring(pidx + 1);
+    res = +s.substring(0,pidx);
+    return res * Math.pow(2,exp);
+  }
   caml_failwith("float_of_string");
 }
 
@@ -378,7 +456,7 @@ function caml_parse_format (fmt) {
 }
 
 //Provides: caml_finish_formatting
-//Requires: MlString
+//Requires: MlWrappedString
 function caml_finish_formatting(f, rawbuffer) {
   if (f.uppercase) rawbuffer = rawbuffer.toUpperCase();
   var len = rawbuffer.length;
@@ -407,7 +485,7 @@ function caml_finish_formatting(f, rawbuffer) {
 }
 
 //Provides: caml_format_int const
-//Requires: caml_parse_format, caml_finish_formatting, MlString
+//Requires: caml_parse_format, caml_finish_formatting, MlWrappedString,caml_str_repeat
 function caml_format_int(fmt, i) {
   if (fmt.toString() == "%d") return new MlWrappedString(""+i);
   var f = caml_parse_format(fmt);
@@ -484,7 +562,7 @@ function caml_hash_univ_param (count, limit, obj) {
         // Object
         count --;
         hash_accu = (hash_accu * 65599 + obj[2]) | 0;
-        break
+        break;
       case 250:
         // Forward
         limit++; hash_aux(obj); break;
@@ -524,7 +602,7 @@ function caml_hash_univ_param (count, limit, obj) {
 }
 
 //Provides: caml_hash mutable
-//Requires: MlString, caml_int64_bits_of_float
+//Requires: MlString, caml_int64_bits_of_float, caml_mul
 var caml_hash =
 function () {
   var HASH_QUEUE_SIZE = 256;
@@ -663,7 +741,7 @@ function () {
 var caml_initial_time = new Date() * 0.001;
 function caml_sys_time () { return new Date() * 0.001 - caml_initial_time; }
 //Provides: caml_sys_get_config const
-//Requires: MlString
+//Requires: MlWrappedString
 function caml_sys_get_config () {
   return [0, new MlWrappedString("Unix"), 32, 0];
 }
@@ -677,6 +755,12 @@ function caml_sys_random_seed () {
 function caml_sys_const_big_endian () { return 0; }
 //Provides: caml_sys_const_word_size const
 function caml_sys_const_word_size () { return 32; }
+//Provides: caml_sys_const_int_size const
+function caml_sys_const_int_size () { return 32; }
+
+//Provides: caml_sys_const_max_wosize const
+function caml_sys_const_max_wosize () { return 0x7FFFFFFF; /* max_int */}
+
 //Provides: caml_sys_const_ostype_cygwin const
 function caml_sys_const_ostype_cygwin () { return 0; }
 //Provides: caml_sys_const_ostype_unix const
@@ -698,7 +782,7 @@ function caml_array_append(a1, a2) {
 //Provides: caml_array_concat mutable
 function caml_array_concat(l) {
   var a = [0];
-  while (l != 0) {
+  while (l !== 0) {
     var b = l[1];
     for (var i = 1; i < b.length; i++) a.push(b[i]);
     l = l[2];
@@ -717,36 +801,30 @@ function caml_array_blit(a1, i1, a2, i2, len) {
 
 ///////////// CamlinternalOO
 //Provides: caml_get_public_method const
-function caml_get_public_method (obj, tag) {
+var caml_method_cache = [];
+function caml_get_public_method (obj, tag, cacheid) {
   var meths = obj[1];
+  var ofs = caml_method_cache[cacheid];
+  if (ofs === null) {
+    // Make sure the array is not sparse
+    for (var i = caml_method_cache.length; i < cacheid; i++)
+      caml_method_cache[i] = 0;
+  } else if (meths[ofs] === tag) {
+//      console.log("cache hit");
+    return meths[ofs - 1];
+  }
+//  console.log("cache miss");
   var li = 3, hi = meths[1] * 2 + 1, mi;
   while (li < hi) {
     mi = ((li+hi) >> 1) | 1;
     if (tag < meths[mi+1]) hi = mi-2;
     else li = mi;
   }
+  caml_method_cache[cacheid] = li + 1;
   /* return 0 if tag is not there */
   return (tag == meths[li+1] ? meths[li] : 0);
 }
 
-/////////////////////////////
-
-// Dummy functions
-//Provides: caml_ml_out_channels_list const
-function caml_ml_out_channels_list () { return 0; }
-//Provides: caml_ml_flush const
-function caml_ml_flush () { return 0; }
-//Provides: caml_ml_open_descriptor_out const
-function caml_ml_open_descriptor_out () { return 0; }
-//Provides: caml_ml_open_descriptor_in const
-function caml_ml_open_descriptor_in () { return 0; }
-//Provides: caml_sys_get_argv const
-//Requires: MlString
-function caml_sys_get_argv () {
-  var p = new MlWrappedString("a.out"); return [0, p, [0, p]];
-}
-//Provides: caml_ml_output const
-function caml_ml_output () { return 0; }
 //Provides: caml_final_register const
 function caml_final_register () { return 0; }
 //Provides: caml_final_release const
@@ -754,20 +832,70 @@ function caml_final_release () { return 0; }
 //Provides: caml_backtrace_status const
 function caml_backtrace_status () { return 0; }
 //Provides: caml_get_exception_backtrace const
-function caml_get_exception_backtrace () {
-  caml_invalid_argument
-    ("Primitive 'caml_get_exception_backtrace' not implemented");
-}
+function caml_get_exception_backtrace () { return 0; }
 //Provides: caml_get_exception_raw_backtrace const
-function caml_get_exception_raw_backtrace () {
-  caml_invalid_argument
-    ("Primitive 'caml_get_exception_raw_backtrace' not implemented");
-}
+function caml_get_exception_raw_backtrace () { return 0; }
+//Provides: caml_record_backtrace
+function caml_record_backtrace () { return 0; }
 //Provides: caml_convert_raw_backtrace const
-function caml_convert_raw_backtrace () {
-  caml_invalid_argument
-    ("Primitive 'caml_convert_raw_backtrace' not implemented");
-}
+function caml_convert_raw_backtrace () { return 0; }
+//Provides: caml_get_current_callstack const
+function caml_get_current_callstack () { return 0; }
 //Provides: caml_sys_getenv
 //Requires: caml_raise_not_found
-function caml_sys_getenv () { caml_raise_not_found (); }
+//Requires: MlWrappedString
+function caml_sys_getenv (name) {
+  var g = joo_global_object;
+  var n = name.toString();
+  //nodejs env
+  if(g.process
+     && g.process.env
+     && g.process.env[n] != undefined)
+    return new MlWrappedString(g.process.env[n]);
+  caml_raise_not_found ();
+}
+//Provides: caml_sys_exit
+//Requires: caml_invalid_argument
+function caml_sys_exit (code) {
+  var g = joo_global_object;
+  if(g.quit) g.quit(code);
+  //nodejs
+  if(g.process && g.process.exit) g.process.exit(code);
+  caml_invalid_argument("Function 'exit' not implemented");
+}
+
+//Provides: caml_sys_get_argv const
+//Requires: MlWrappedString
+function caml_sys_get_argv () {
+  var g = joo_global_object;
+  var main = "a.out";
+  var args = []
+
+  if(g.process
+     && g.process.argv
+     && g.process.argv.length > 0) {
+    //nodejs
+    main = g.process.argv[0];
+    args = g.process.argv.slice(1);
+  }
+
+  var p = new MlWrappedString(main);
+  var args2 = [0, p];
+  for(var i = 0; i < args.length; i++)
+    args2.push(new MlWrappedString(args[i]));
+  return [0, p, args2];
+}
+
+//Provides: unix_inet_addr_of_string
+function unix_inet_addr_of_string () {return 0;}
+
+
+//Provides: caml_set_oo_id
+var caml_oo_last_id = 0;
+function caml_set_oo_id (b) {
+  b[2]=caml_oo_last_id++;
+  return b;
+}
+
+//Provides: caml_install_signal_handler const
+function caml_install_signal_handler(){return 0}
